@@ -7,163 +7,176 @@ from collections import deque
 from typing import Tuple, List, Callable, Any, Dict, Optional
 import os
 
-import requests
-import time
 import threading
-import uuid
+import time
+import requests
 import json
-import traceback
+import uuid
 
-# 配置信息 - 替换为您的实际值
-SERVER_URL = "https://scrawl.weber.edu.deal/api"  # 您的 Vercel 部署 URL
-
-
-class CloudVariableClient:
-    def __init__(self, server_url=SERVER_URL, project_id=None, api_key=None):
-        self.server_url = server_url
-        self.project_id = project_id
-        self.api_key = api_key
-        self.variables = {}
+class CloudVariablesClient:
+    def __init__(self, project_name=None, project_id=None, api_key=None, 
+                 base_url="http://scrawl.pythonanywhere.com", sync_interval=100):
+        self.base_url = base_url
+        self.sync_interval = sync_interval/1000 # 毫秒化秒
+        self.local_vars = {}              # 本地变量存储
+        self.change_queue = deque()       # 变更队列 (线程安全)
+        self.lock = threading.Lock()      # 线程锁
+        self.running = True               # 控制同步线程运行
         
-        # 如果没有提供项目ID，自动注册新项目
-        if not project_id or not api_key:
-            self.register_project()
-    
-    def register_project(self, project_name="TestProject"):
-        """注册新项目"""
-        url = f"{self.server_url}/register"
-        payload = {"project_name": project_name}
+        # 注册新项目或使用现有项目
+        if project_name:
+            self._register_project(project_name)
+        elif project_id and api_key:
+            self.project_id = project_id
+            self.api_key = api_key
+        else:
+            raise ValueError("必须提供project_name或project_id/api_key")
+        
+        # 从服务器加载所有变量
+        self._load_all_variables()
+        
+        # 启动同步线程
+        self.sync_thread = threading.Thread(target=self._sync_loop, daemon=True)
+        self.sync_thread.start()
+
+    def _register_project(self, project_name):
+        """注册新项目并获取凭证"""
+        url = f"{self.base_url}/api/register"
+        try:
+            response = requests.post(url, json={'project_name': project_name})
+            response.raise_for_status()
+            data = response.json()
+            self.project_id = data['project_id']
+            self.api_key = data['api_key']
+            print(f"项目注册成功! ID: {self.project_id}, API Key: {self.api_key}")
+        except requests.exceptions.RequestException as e:
+            raise ConnectionError(f"连接服务器失败: {str(e)}")
+
+    def _load_all_variables(self):
+        """从服务器加载所有变量到本地"""
+        url = f"{self.base_url}/api/{self.project_id}/all"
+        headers = {'X-API-Key': self.api_key}
         
         try:
-            print(f"📤 注册项目: {project_name}")
-            response = requests.post(url, json=payload, timeout=10)
+            response = requests.get(url, headers=headers)
+            response.raise_for_status()
+            server_vars = response.json()
             
-            if response.status_code == 201:
-                data = response.json()
-                self.project_id = data['project_id']
-                self.api_key = data['api_key']
-                print(f"✅ 项目注册成功! ID: {self.project_id}, API Key: {self.api_key}")
-                return True
-            else:
-                print(f"❌ 项目注册失败: {response.status_code}")
-                print(f"响应内容: {response.text}")
-                print(f"请求URL: {url}")
-                print(f"请求负载: {payload}")
-                return False
-        except Exception as e:
-            print(f"❌ 注册请求异常: {str(e)}")
-            traceback.print_exc()
-            return False
-    
-    def set_variable(self, var_name, var_value):
-        """设置变量值"""
-        if not self.project_id or not self.api_key:
-            print("❌ 未设置项目ID或API密钥")
-            return False
-        
-        url = f"{self.server_url}/{self.project_id}/set"
-        headers = {"X-API-Key": self.api_key}
-        payload = {"var_name": var_name, "var_value": var_value}
-        
-        try:
-            print(f"📤 设置变量: {var_name} = {var_value}")
-            response = requests.post(url, json=payload, headers=headers, timeout=5)
+            with self.lock:
+                self.local_vars = {}
+                for var_name, var_data in server_vars.items():
+                    # 直接存储解析后的值
+                    self.local_vars[var_name] = var_data['value']
+        except requests.exceptions.RequestException:
+            print("警告: 无法从服务器加载初始变量")
+        except json.JSONDecodeError:
+            print("错误: 服务器返回了无效的JSON数据")
+
+    def set_variable(self, var_name, value):
+        """设置变量值（线程安全）"""
+        with self.lock:
+            # 检查值是否实际变化
+            if var_name in self.local_vars and self.local_vars[var_name] == value:
+                return
             
-            if response.status_code == 200:
-                print(f"✅ 设置变量成功: {var_name} = {var_value}")
-                self.variables[var_name] = var_value
-                return True
-            else:
-                print(f"❌ 设置变量失败: {response.status_code}")
-                print(f"响应内容: {response.text}")
-                print(f"请求URL: {url}")
-                print(f"请求头: {headers}")
-                print(f"请求负载: {payload}")
-                return False
-        except Exception as e:
-            print(f"❌ 设置变量异常: {str(e)}")
-            traceback.print_exc()
-            return False
-    
-    def get_variable(self, var_name):
-        """获取变量值"""
-        if not self.project_id or not self.api_key:
-            print("❌ 未设置项目ID或API密钥")
-            return None
-        
-        url = f"{self.server_url}/{self.project_id}/get"
-        headers = {"X-API-Key": self.api_key}
-        params = {"var_name": var_name}
-        
-        try:
-            print(f"📥 获取变量: {var_name}")
-            response = requests.get(url, params=params, headers=headers, timeout=5)
+            # 更新本地值
+            self.local_vars[var_name] = value
             
-            if response.status_code == 200:
-                data = response.json()
-                value = data['var_value']
-                print(f"✅ 获取变量成功: {var_name} = {value}")
-                self.variables[var_name] = value
-                return value
-            else:
-                print(f"❌ 获取变量失败: {response.status_code}")
-                print(f"响应内容: {response.text}")
-                print(f"请求URL: {url}")
-                print(f"请求头: {headers}")
-                print(f"请求参数: {params}")
-                return None
-        except Exception as e:
-            print(f"❌ 获取变量异常: {str(e)}")
-            traceback.print_exc()
-            return None
-    
+            # 添加到变更队列
+            self.change_queue.append((var_name, value))
+
+    def get_variable(self, var_name, default=None):
+        """获取变量值（线程安全）"""
+        with self.lock:
+            return self.local_vars.get(var_name, default)
+
     def get_all_variables(self):
-        """获取所有变量"""
-        if not self.project_id or not self.api_key:
-            print("❌ 未设置项目ID或API密钥")
-            return {}
+        """获取所有变量副本（线程安全）"""
+        with self.lock:
+            return self.local_vars.copy()
+
+    def _sync_changes(self):
+        """同步变更到服务器"""
+        if not self.change_queue:
+            return
         
-        url = f"{self.server_url}/{self.project_id}/all"
-        headers = {"X-API-Key": self.api_key}
+        # 批量准备变更
+        changes = []
+        while self.change_queue:
+            changes.append(self.change_queue.popleft())
+        
+        # 准备批量更新数据
+        update_data = [{
+            'var_name': var_name,
+            'var_value': value
+        } for var_name, value in changes]
+        
+        # 发送批量更新请求
+        headers = {
+            'X-API-Key': self.api_key,
+            'Content-Type': 'application/json'
+        }
+        url = f"{self.base_url}/api/{self.project_id}/batch_update"
         
         try:
-            print("📥 获取所有变量")
-            response = requests.get(url, headers=headers, timeout=5)
-            
-            if response.status_code == 200:
-                variables = response.json()
-                print(f"✅ 获取所有变量成功: 共 {len(variables)} 个变量")
-                self.variables = {k: v['value'] for k, v in variables.items()}
-                return variables
-            else:
-                print(f"❌ 获取所有变量失败: {response.status_code}")
-                print(f"响应内容: {response.text}")
-                return {}
-        except Exception as e:
-            print(f"❌ 获取所有变量异常: {str(e)}")
-            traceback.print_exc()
-            return {}
-    
-    def health_check(self):
-        """健康检查"""
-        url = f"{self.server_url}/health"
+            response = requests.post(
+                url, 
+                headers=headers, 
+                json={'updates': update_data}
+            )
+            response.raise_for_status()
+        except requests.exceptions.RequestException as e:
+            print(f"同步失败: {str(e)}")
+            # 重新加入队列以便重试
+            with self.lock:
+                for change in changes:
+                    self.change_queue.appendleft(change)
+
+    def _fetch_updates(self):
+        """从服务器获取更新"""
+        url = f"{self.base_url}/api/{self.project_id}/all"
+        headers = {'X-API-Key': self.api_key}
         
         try:
-            print("🩺 健康检查")
-            response = requests.get(url, timeout=5)
+            response = requests.get(url, headers=headers)
+            response.raise_for_status()
+            server_vars = response.json()
             
-            if response.status_code == 200:
-                data = response.json()
-                print(f"✅ 健康检查通过: {data}")
-                return True
-            else:
-                print(f"❌ 健康检查失败: {response.status_code}")
-                print(f"响应内容: {response.text}")
-                return False
-        except Exception as e:
-            print(f"❌ 健康检查异常: {str(e)}")
-            traceback.print_exc()
-            return False
+            with self.lock:
+                # 只更新本地未修改的变量
+                for var_name, var_data in server_vars.items():
+                    # 检查此变量是否有未同步的本地修改
+                    has_pending_change = any(
+                        name == var_name for name, _ in self.change_queue
+                    )
+                    
+                    if not has_pending_change:
+                        self.local_vars[var_name] = var_data['value']
+        except requests.exceptions.RequestException:
+            pass  # 网络错误时静默失败
+
+    def _sync_loop(self):
+        """同步循环：定期执行同步"""
+        while self.running:
+            try:
+                # 第一步：推送本地变更
+                self._sync_changes()
+                
+                # 第二步：获取服务器更新
+                self._fetch_updates()
+            except Exception as e:
+                print(f"同步错误: {str(e)}")
+            
+            time.sleep(self.sync_interval)
+
+    def close(self):
+        """关闭客户端并停止同步"""
+        self.running = False
+        self.sync_thread.join(timeout=2.0)
+        
+        # 最后尝试同步所有变更
+        self._sync_changes()
+        print("云变量客户端已关闭")
 
 # 获取当前包目录的绝对路径
 PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))
